package java.io;

public class StringBufferInputStream extends InputStream {
   protected String buffer;
   protected int pos;
   protected int count;

   public StringBufferInputStream(String s) {
      this.buffer = s;
      this.count = s.length();
   }

   public synchronized int read() {
      return this.pos < this.count ? this.buffer.charAt(this.pos++) : -1;
   }

   public synchronized int read(byte[] b, int off, int len) {
      if (this.pos >= this.count) {
         return -1;
      } else {
         if (this.pos + len > this.count) {
            len = this.count - this.pos;
         }

         if (len <= 0) {
            return 0;
         } else {
            String s = this.buffer;
            int cnt = len;

            while(true) {
               --cnt;
               if (cnt < 0) {
                  return len;
               }

               b[off++] = (byte)s.charAt(this.pos++);
            }
         }
      }
   }

   public synchronized long skip(long n) {
      if ((long)this.pos + n > (long)this.count) {
         n = (long)(this.count - this.pos);
      }

      if (n < 0L) {
         return 0L;
      } else {
         this.pos = (int)((long)this.pos + n);
         return n;
      }
   }

   public synchronized int available() {
      return this.count - this.pos;
   }

   public synchronized void reset() {
      this.pos = 0;
   }
}
